from baby_steps import then, when

from vedro.plugins.director import Reporter, RichReporter


def test_rich_reporter():
    with when:
        reporter = RichReporter()

    with then:
        assert isinstance(reporter, Reporter)



from unittest.mock import Mock, call

import pytest
from baby_steps import given, then, when
from rich.console import Style

from vedro.core import Dispatcher, ScenarioResult
from vedro.events import ScenarioPassedEvent
from vedro.plugins.director import RichReporter
from vedro.plugins.director.rich.test_utils import console_, dispatcher, make_vscenario, reporter

__all__ = ("dispatcher", "reporter", "console_",)


@pytest.mark.asyncio
async def test_rich_reporter_scenario_pass_event(*, dispatcher: Dispatcher,
                                                 reporter: RichReporter, console_: Mock):
    with given:
        reporter.subscribe(dispatcher)

        subject = "<subject>"
        vscenario = make_vscenario(subject=subject)
        scenario_result = ScenarioResult(vscenario).mark_passed()
        event = ScenarioPassedEvent(scenario_result)

    with when:
        await dispatcher.fire(event)

    with then:
        assert console_.mock_calls == [
            call.out(f" ✔ {subject}", style=Style.parse("green"))
        ]



import asyncio
from asyncio import CancelledError
from typing import List, Optional

from ._context import context
from ._core import Dispatcher, Lifecycle, Plugin, Runner, ScenarioDiscoverer
from ._core._scenario_finder import ScenarioFileFinder
from ._core._scenario_finder._file_filters import AnyFilter, DunderFilter, ExtFilter, HiddenFilter
from ._core._scenario_loader import ScenarioAssertRewriterLoader
from ._interface import Interface
from ._params import params
from ._scenario import Scenario
from ._version import version
from .plugins.deferrer import Deferrer, defer
from .plugins.director import Director, PyCharmReporter, Reporter, RichReporter, SilentReporter
from .plugins.interrupter import Interrupter
from .plugins.seeder import Seeder
from .plugins.skipper import Skipper, only, skip
from .plugins.slicer import Slicer
from .plugins.tagger import Tagger
from .plugins.terminator import Terminator

__version__ = version
__all__ = ("Scenario", "Interface", "Runner", "run", "only", "skip", "params",
           "context", "defer",)


def run(*, plugins: Optional[List[Plugin]] = None) -> None:
    finder = ScenarioFileFinder(
        file_filter=AnyFilter([
            HiddenFilter(),
            DunderFilter(),
            ExtFilter(only=["py"]),
        ]),
        dir_filter=AnyFilter([
            HiddenFilter(),
            DunderFilter(),
        ])
    )
    loader = ScenarioAssertRewriterLoader()
    discoverer = ScenarioDiscoverer(finder, loader)
    dispatcher = Dispatcher()

    all_plugins = []
    reporters = [
        RichReporter(),
        SilentReporter(),
        PyCharmReporter(),
    ]

    if plugins:
        for plugin in plugins:
            if isinstance(plugin, Reporter):
                reporters.append(plugin)
            else:
                all_plugins.append(plugin)

    all_plugins += [
        Director(reporters),
        Seeder(),
        Skipper(),
        Tagger(),
        Slicer(),
        Interrupter(),
        Deferrer(),
        Terminator(),
    ]

    runner = Runner(dispatcher, (KeyboardInterrupt, SystemExit, CancelledError,))
    lifecycle = Lifecycle(dispatcher, discoverer, runner)
    lifecycle.register_plugins(all_plugins)

    asyncio.run(lifecycle.start())



from .._core import (
    Dispatcher,
    ExcInfo,
    Lifecycle,
    Plugin,
    Report,
    Runner,
    ScenarioResult,
    ScenarioStatus,
    StepResult,
    StepStatus,
    Subscriber,
    VirtualScenario,
    VirtualStep,
)

__all__ = ("Dispatcher", "Subscriber", "Plugin", "ScenarioResult", "StepResult",
           "ScenarioStatus", "StepStatus", "VirtualScenario", "VirtualStep",
           "ExcInfo", "Report", "Runner", "Lifecycle",)



import sys
from traceback import format_exception
from typing import Any, Dict, TextIO

from vedro.core import Dispatcher, ExcInfo, ScenarioResult
from vedro.events import (
    ScenarioFailedEvent,
    ScenarioPassedEvent,
    ScenarioRunEvent,
    ScenarioSkippedEvent,
)

from .._reporter import Reporter

__all__ = ("PyCharmReporter",)


class PyCharmReporter(Reporter):
    def __init__(self, out: TextIO = sys.stdout) -> None:
        self._out = out

    @property
    def name(self) -> str:
        return "pycharm"

    def subscribe(self, dispatcher: Dispatcher) -> None:
        dispatcher.listen(ScenarioRunEvent, self.on_scenario_run) \
                  .listen(ScenarioPassedEvent, self.on_scenario_passed) \
                  .listen(ScenarioFailedEvent, self.on_scenario_failed) \
                  .listen(ScenarioSkippedEvent, self.on_scenario_skipped)

    def on_scenario_run(self, event: ScenarioRunEvent) -> None:
        scenario_result: ScenarioResult = event.scenario_result
        self._write_message("testStarted", {
            "name": scenario_result.scenario.subject,
        })

    def on_scenario_passed(self, event: ScenarioPassedEvent) -> None:
        scenario_result: ScenarioResult = event.scenario_result
        self._write_message("testFinished", {
            "name": scenario_result.scenario.subject,
            "duration": int(scenario_result.elapsed * 1000),
        })

    def on_scenario_failed(self, event: ScenarioFailedEvent) -> None:
        scenario_result: ScenarioResult = event.scenario_result

        message = ""
        for step_result in scenario_result.step_results:
            if step_result.exc_info:
                message += self._format_exception(step_result.exc_info)

        self._write_message("testFailed", {
            "name": scenario_result.scenario.subject,
            "duration": int(scenario_result.elapsed * 1000),
            "message": message,
        })

    def on_scenario_skipped(self, event: ScenarioSkippedEvent) -> None:
        scenario_result: ScenarioResult = event.scenario_result
        self._write_message("testIgnored", {
            "name": scenario_result.scenario.subject,
        })

    def _format_exception(self, exc_info: ExcInfo) -> str:
        formatted = format_exception(exc_info.type, exc_info.value, exc_info.traceback)
        return "".join(formatted)

    def _escape_value(self, value: str) -> str:
        symbols = {"'": "|'", "\n": "|n", "\r": "|r", "|": "||", "[": "|[", "]": "|]"}
        return value.translate({ord(k): v for k, v in symbols.items()})

    def _write_message(self, name: str, attributes: Dict[str, Any]) -> None:
        message = f"##teamcity[{name}"
        for key, val in attributes.items():
            escaped = self._escape_value(str(val))
            message += f" {key}='{escaped}'"
        message += "]\n"
        self._out.write(message)
